import argparse
import platform
from pathlib import Path
from typing import Optional, Sequence

from pydantic import BaseModel
from yarl import URL


def get_default_dir() -> Path:
    platform_specific = "frage" if platform.system() == "Windows" else ".frage"
    return Path.home() / platform_specific / "requests"


class Config(BaseModel):
    request_name: str
    base_url: Optional[URL]
    dir: Path

    class Config:
        arbitrary_types_allowed = True


def get_config(raw_args: Optional[Sequence[str]] = None) -> Config:
    args = parse_args(raw_args)
    return Config(
        request_name=args.name,
        base_url=args.base_url,
        dir=args.dir or get_default_dir(),
    )


def parse_args(raw_args: Optional[Sequence[str]] = None):
    parser = argparse.ArgumentParser(description="Make configurable HTTP requests")
    parser.add_argument(
        "name", help="Name of Dhall file with request definition, without extension"
    )
    parser.add_argument(
        "-u", "--base-url", type=URL, help="Base URL to be joined with request path"
    )
    parser.add_argument(
        "-d", "--dir", type=Path, help="Directory where request files will be searched"
    )

    return parser.parse_args(raw_args)



from enum import Enum

from pydantic import BaseModel, AnyHttpUrl


class Method(str, Enum):
    HEAD = "HEAD"
    OPTIONS = "OPTIONS"
    GET = "GET"
    POST = "POST"
    PUT = "PUT"
    PATCH = "PATCH"
    DELETE = "DELETE"

    TRACE = "TRACE"
    CONNECT = "CONNECT"


Headers = dict[str, str]
Body = str


class Request(BaseModel):
    path: str
    method: Method
    headers: Headers
    body: Body


class Response(BaseModel):
    status: int
    headers: Headers
    body: Body



from typing import Optional

from aiohttp import ClientSession
from yarl import URL

from frage.models import Request, Response


async def make_request(
    session: ClientSession,
    request: Request,
    *,
    base_url: Optional[URL] = None,
) -> Response:
    request_url = URL(request.path)
    url = base_url.join(request_url) if base_url else request_url

    req = session.request(
        request.method,
        url,
        headers=request.headers,
        data=request.body,
    )
    async with req as resp:
        response = Response(
            status=resp.status,
            headers=resp.headers,
            body=await resp.read(),
        )

    return response



from contextlib import contextmanager
from pathlib import Path
from typing import TextIO, Iterator

import dhall

from frage.models import Request


class ParsingError(Exception):
    pass


@contextmanager
def open_file(dir_: Path, name: str) -> Iterator[TextIO]:
    with (dir_ / Path(name + ".dhall")).open() as f:
        yield f


def parse(data: TextIO) -> Request:
    try:
        return Request.parse_obj(dhall.load(data))
    except Exception as e:
        raise ParsingError() from e



import sys
import asyncio
from typing import IO

from aiohttp import ClientSession

from frage.config import get_config
from frage.http import make_request
from frage.models import Response
from frage.parser import open_file, parse


def output(response: Response, stream: IO):
    stream.write(response.body)


async def main():
    config = get_config()

    with open_file(config.dir, config.request_name) as f:
        request = parse(f)

    async with ClientSession() as s:
        response = await make_request(s, request, base_url=config.base_url)

    output(response, sys.stdout)


def run():
    asyncio.run(main())


if __name__ == "__main__":  # pragma: nocover
    run()



# -*- coding: utf-8 -*-
from setuptools import setup

packages = \
['frage']

package_data = \
{'': ['*']}

install_requires = \
['aiohttp>=3.7.4,<4.0.0', 'dhall>=0.1.7,<0.2.0', 'pydantic>=1.8.2,<2.0.0']

entry_points = \
{'console_scripts': ['frage = frage.__main__:run']}

setup_kwargs = {
    'name': 'frage',
    'version': '0.2.0',
    'description': 'Make pre-configured requests from the commandline',
    'long_description': 'About\n---\nMake pre-configured requests from the command line.\n\n\nAuthor\n---\n\nElias Tandel < elias.tandel at gmail >\n',
    'author': 'Elias Tandel',
    'author_email': 'elias.tandel@gmail.com',
    'maintainer': None,
    'maintainer_email': None,
    'url': 'https://github.com/etandel/frage',
    'packages': packages,
    'package_data': package_data,
    'install_requires': install_requires,
    'entry_points': entry_points,
    'python_requires': '>=3.9,<4.0',
}


setup(**setup_kwargs)
